import io
import sys

import pyomo.environ as pyo
from ormm.opt.mathprog.problems import ResourceAllocation, print_sol

def create_cm_with_data():
    """Use exmaple data & return results and solved instance"""
    data_path = "../ormm/opt/mathprog/problems/example_data/resource_allocation.dat"
    mod = ResourceAllocation()
    instance = mod.create_instance(data_path)
    opt = pyo.SolverFactory("glpk")
    results = opt.solve(instance)
    return instance, results

def test_resource_allocation_with_data():
    instance, results = create_cm_with_data()
    # Check all variable values
    for v in instance.component_objects(pyo.Var, active=True):
        if v.name == "NumActivity":
            print({index: v[index].value for index in v})
            assert {index: v[index].value for index in v} == {"P": 100, "Q": 30}
    assert instance.OBJ() == 6300
def test_print_sol_with_data():
    instance, results = create_cm_with_data()
    # Redirect output to StringIO object
    captured_output = io.StringIO()
    sys.stdout = captured_output
    print_sol(instance)
    sys.stdout = sys.__stdout__  # reset stdout
    test_string = (
        "Objective Value: $6,300.0\n"
        "Variable component:  NumActivity\n"
        "    P 100.0\n"
        "    Q 30.0\n"
    )
    assert captured_output.getvalue() == test_string


import pyomo.environ as pyo

class ResourceAllocation(pyo.AbstractModel):
    """Pyomo Abstract Model for the Resource Allocation Problem

    :return: A pyomo abstract model with definitions for the Resource Allocation Problem

    See Also
    --------
    Formulation: Documentation with definitions & details of the abstract model.

    Examples
    --------
    Creating abstract model, an instance from data params, & solving instance.

    >>> model = ResourceAllocation()
    >>> instance = model.create_instance(my_params.dat) # AMPL data file
    >>> opt = pyo.SolverFactory("glpk")
    >>> opt.solve(instance)
    """

    def __init__(self):
        """Constructor method - calls create_abstract_model
        """
        super().__init__()
        self.create_abstract_model()

    def create_abstract_model(self):
        """Create the abstract model for Resource Allocation Problem
        """
        self.Activities = pyo.Set()
        self.Resources = pyo.Set()
        self.Values = pyo.Param(self.Activities)
        self.ResourceNeeds = pyo.Param(self.Resources, self.Activities)
        self.MaxResource = pyo.Param(self.Resources)
        self.MaxActivity = pyo.Param(self.Activities)
        self.NumActivity = pyo.Var(self.Activities, within = pyo.NonNegativeIntegers, bounds = self._get_bounds)

        self.OBJ = pyo.Objective(rule = self._obj_expression, sense = pyo.maximize)
        self.ResourceConstraint = pyo.Constraint(self.Resources, rule = self._resource_constraint_rule)

    def _get_bounds(self, model, p):
        """Upper Bounds for the Decision Vars based on Maximum Demand
        """
        return (0, self.MaxActivity[p])

    def _obj_expression(self, model):
        """Objective Expression: Maximizing Profit
        """
        return pyo.summation(self.Values, self.NumActivity)

    def _resource_constraint_rule(self, model, m):
        """Constraints for Scarce Resources"""
        return sum(self.ResourceNeeds[m, p] * self.NumActivity[p] for p in self.Activities) <= self.MaxResource[m]

def print_sol(instance):
    """Print the solution to the solved `instance`
    :param instance: A solved pyomo.environ.ConcreteModel
    """
    print(f"Objective Value: ${instance.OBJ():,}")
    for v in instance.component_objects(pyo.Var, active=True):
        print ("Variable component: ",v)
        for index in v:
            print ("   ", index, v[index].value)


from setuptools import setup, find_packages

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name = "ormm",
    version = "0.0.1",
    description = "Operations Research Models & Methods",
    url = "https://github.com/egbuck/ormm",
    author = "Ethan Buck",
    author_email = "egbuck96@gmail.com",
    packages = find_packages(include=["ormm", "ormm.*"]),
    long_description = long_description,
    long_description_content_type = "text/x-rst",
    install_requires = [
        "pyomo >= 5.0",
    ],
    extras_require = {
        "dev": [
            "pytest >= 6.0",
            "sphinx >= 3.1.2",
            "sphinx_rtd_theme >= 0.5.0",
            "numpydoc >= 1.1.0",
            "twine >= 3.2.0",
            #"check-manifest>=0.42" # used for creating Manifest.in
        ]
    },
    classifiers = [
        "Programming Language :: Cython",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Manufacturing",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Education",
        "Topic :: Education",
        "Topic :: Scientific/Engineering :: Mathematics",
    ],
)